/* ===========================================================================================================
   cpr-rescue.css
   Styles for the CPR Rescue WordPress plugin
   =========================================================================================================== */

/* --- About table --- */
.cpr-about-table {
    width: 100%;
    border-collapse: collapse;
}

.cpr-about-table td {
    padding: 5px;
    vertical-align: top;
}

.cpr-about-table td:first-child {
    width: 40%;
    white-space: nowrap;
}

.cpr-about-table td:last-child {
    width: 60%;
}

/* --- Animal list grid --- */
.cpr-animal-thumb {
    max-height: 400px;
    width: 100%;
    object-fit: cover;
}

/* --- Carousel --- */
.cpr-carousel-img {
    height: 500px;
    object-fit: contain;
}

.cpr-video-slide {
    height: 500px;
    background: #000;
}

.cpr-video-iframe {
    width: 100%;
    height: 500px;
}

.cpr-thumb {
    max-height: 100px;
    width: 100%;
    object-fit: cover;
    cursor: pointer;
}

.cpr-no-photo {
    height: 500px;
}

/* Thumbnail strip hidden on mobile, shown on md and above */
@media (max-width: 767px) {
    .cpr-carousel-img {
        height: 300px;
    }
    .cpr-video-slide,
    .cpr-video-iframe {
        height: 300px;
    }
    .cpr-no-photo {
        height: 300px;
    }
}

/* --- Panel action buttons --- */
.cpr-action-btn {
    float: right;
    font-size: 1.5rem;
    margin-top: 10px;
}
